/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.javable.cese.models;

import java.awt.Image;
import java.beans.BeanDescriptor;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.beans.SimpleBeanInfo;

public class HodgkinHuxleyModel_1952BeanInfo extends SimpleBeanInfo {
    
  /** Icon for image data objects. */
    private Image icon;
    
  /** Extended descriptor for this bean. */    
    private BeanDescriptor descriptor;
    
  /** Array of property descriptors. */
    public HodgkinHuxleyModel_1952BeanInfo() {
        icon = loadImage ("model_neuronal.png");
    }
    
  /** Provides the Models's icon */
    public Image getIcon(int type) {
        return icon;
    }
    
    /** Provides the Models's descriptor */    
    public BeanDescriptor getBeanDescriptor(){
    	descriptor = new BeanDescriptor(HodgkinHuxleyModel_1952.class);
    	descriptor.setDisplayName("The Hodgkin-Huxley Squid Axon Model, 1952");
        descriptor.setShortDescription(
        "The Hodgkin-Huxley Squid Axon Model, 1952 <p>"+
        "J Physiol 1952;117:500-544.");
    	
        descriptor.setValue("version", "1.4");
    	return descriptor ;
    }
    
  /** Create a new property descriptor
   * @return Property Descriptor
   */
    private PropertyDescriptor createPropertyDescriptor (String name, String gett, String sett, String group) {
        PropertyDescriptor pd=null;
        try {
            pd = new PropertyDescriptor (name, HodgkinHuxleyModel_1952.class, gett, sett);
            pd.setShortDescription(group);
        }
        catch (IntrospectionException ex) {}
        return pd;
    }
    
  /** Descriptor of valid properties
   * @return array of properties
   */
    public PropertyDescriptor[] getPropertyDescriptors () {
        PropertyDescriptor[] desc =
        {
            createPropertyDescriptor ("Length of the cell (cm)",  "getl", "setl","Cell Geometry"),
            createPropertyDescriptor ("Radius of the cell (cm)",  "geta", "seta","Cell Geometry"),
            
            createPropertyDescriptor ("Membrane voltage (mV)",  "getv", "setv","Voltage"),
            
            createPropertyDescriptor ("Time step (ms)",  "getdt", "setdt","Time Step"),
            createPropertyDescriptor ("Initial time (ms)",  "gett", "sett","Time Step"),
            
            createPropertyDescriptor ("Stimulus amplitude (uA/cm^2)",  "getst", "setst","Total Current and Stimulus"),
            createPropertyDescriptor ("Time when stimulus is applied (ms)",  "gettstim", "settstim","Total Current and Stimulus"),
            createPropertyDescriptor ("Stimulus duration (ms)",  "getstl", "setstl","Total Current and Stimulus"),
            createPropertyDescriptor ("Total experiment time (ms)",  "getetime", "setetime","Total Current and Stimulus"),
            createPropertyDescriptor ("Basic cycle length (ms)",  "getbcl", "setbcl","Total Current and Stimulus"),
            
            createPropertyDescriptor ("Temperature (K)",  "gettemp", "settemp","Termodynamics"),
            createPropertyDescriptor ("Reversal potential (mV)",  "geter", "seter","Termodynamics"),
            
            createPropertyDescriptor ("Scaling factor of the Na current",  "getscgna", "setscgna","Fast Sodium Current (time dependent)"),
              
            createPropertyDescriptor ("Scaling factor of the Time-dependent K current",  "getscgkt", "setscgkt","Time-dependent Potassium Current"),
            
            createPropertyDescriptor ("Scaling factor of the Leakage current",  "getscgl", "setscgl","Leakage Current"),
            
            
            createPropertyDescriptor ("|OUT Membrane voltage (mV)",  "getv", null,"Voltage"),
            createPropertyDescriptor ("|OUT d voltage / d time (mV/ms)",  "getdvdt", null,"Voltage"),
            
            createPropertyDescriptor ("|OUT Fast Na current (uA/uF)",  "getina", null,"Fast Sodium Current (time dependent)"),
            createPropertyDescriptor ("|OUT Reversal potential of Na (mV)",  "getena", null,"Fast Sodium Current (time dependent)"),
            createPropertyDescriptor ("|OUT Na activation m",  "getm", null,"Fast Sodium Current (time dependent)"),
            createPropertyDescriptor ("|OUT Na inactivation h",  "geth", null,"Fast Sodium Current (time dependent)"),
                 
            createPropertyDescriptor ("|OUT Time-dependent K current (uA/uF)",  "getikt", null,"Time-dependent Potassium Current"),
            createPropertyDescriptor ("|OUT K time-dependent activation",  "getn", null,"Time-dependent Potassium Current"),
            
            createPropertyDescriptor ("|OUT Leakage current (uA/uF)",  "getil", null,"Leakage Current"),
            
            createPropertyDescriptor ("|OUT Total current (uA/cm^2)",  "getit", null,"Total Current"),
            createPropertyDescriptor ("|OUT Total Na ion flow (mM/ms)",  "getnaiont", null,"Total Current"),
            createPropertyDescriptor ("|OUT Total K ion flow (mM/ms)",  "getkiont", null,"Total Current"),
              
            createPropertyDescriptor ("Number of iterations",  "getsteps", null,"Time Step"),
        };
        return desc;
    }
}