/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.specific;

/**
 * this class describes the way you build the name of an element
 * 
 * @author Sebastien Ponce
 */
public class ElementName {

    /**
     * identify that a name of a given element should be its type
     */
    public static final int TYPE_NAME = 0;

    /**
     * identify that a name of a given element should be its value
     */
    public static final int VALUE_NAME = 1;

    /**
     * identify that a name of a given element should be one of its attributes
     */
    public static final int ATTRIBUTE_NAME = 2;

    /**
     * this gives the type of the name. It could take only 3 values : TYPE_NAME,
     * VALUE_NAME and ATTRIBUTE_NAME
     */
    public int nameType;

    /**
     * this is only used in case nameType is ATTRIBUTE_NAME it gives the name of
     * the attribute concerned
     */
    public String attributeName;

    /**
     * the constructor
     * 
     * @param nameType the initial value for nameType
     * @param attributeName the initial value for AttributeName
     */
    public ElementName(int nameType, String attributeName) {
        this.nameType = nameType;
        this.attributeName = attributeName;
    }

}
