/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import com.simulogic.cese.CESE;
import com.simulogic.cese.ModelManager;
import com.simulogic.cese.ResourceManager;
import com.simulogic.utils.ActionSupport;
import com.simulogic.utils.ExceptionDialog;

/**
 * NewParams action implementation
 */
public class NewParamsAction extends ActionSupport implements
        java.beans.PropertyChangeListener {

    private ModelManager manager;

    /** 
     * Creates new NewParamsAction  
     *  
     * @param m model manager
     */
    public NewParamsAction(ModelManager m) {
        this();
        manager = m;
        manager.addPropertyChangeListener(this);
    }

    /** Creates new NewParamsAction */
    public NewParamsAction() {
        super(ResourceManager.getResource("New_Parameters..."), ResourceManager
                .getResource("New_parameters"), ResourceManager
                .getResource("new_params_image"));
    }

    public void actionPerformed(ActionEvent evt) {
        try {
            int n = JOptionPane.showConfirmDialog(CESE.getEnvironment(),
                    ResourceManager.getResource("Initialize_model?"),
                    ResourceManager.getResource("New_parameters"),
                    JOptionPane.YES_NO_OPTION);
            if (n == JOptionPane.NO_OPTION) return;
            manager.newModel();
        } catch (Exception e) {
            ExceptionDialog.showExceptionDialog(ResourceManager
                    .getResource("Error"), ResourceManager
                    .getResource("Error_initializing_model."), e);
        }
    }

    public void propertyChange(java.beans.PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals("model_control"))
            this.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
    }
}