/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;

import javax.swing.beanbuilder.CustomObjectInputStream;
import javax.swing.beanbuilder.JarClassLoader;

import com.simulogic.cese.CESE;
import com.simulogic.cese.ModelManager;
import com.simulogic.cese.ResourceManager;
import com.simulogic.utils.ActionSupport;
import com.simulogic.utils.ExceptionDialog;

/**
 * OpenRecentParams action implementation
 */
public class OpenRecentParamsAction extends ActionSupport {

	private File file;

	private ModelManager manager;

	/** Creates new OpenRecentParamsAction */
	public OpenRecentParamsAction() {
		super("");
	}

	public void actionPerformed(ActionEvent evt) {
		ObjectInputStream in = null;
		manager = CESE.getEnvironment().getModelManager();

		try {
			file = new File(this.getName());
			if (file.getPath() == null)
					return;

				in = new CustomObjectInputStream(new FileInputStream(file
						.getPath()), JarClassLoader.getJarClassLoader());

				Object lm = in.readObject();
				
				manager.getModelInfo().getHistory().setTreeSelection(
						manager.getExportTreeSelection());
		
				manager.loadModel(lm);
				manager.getModelInfo().getHistory().setParametersFile(file);
				
				manager.getModelInfo().setModified(false);
				manager.changeModel();
		} catch (Exception e) {
			ExceptionDialog.showExceptionDialog(ResourceManager
					.getResource("Error"), ResourceManager
					.getResource("Error_loading_parameters_file."), e);
		} finally {
			try {
				in.close();
			} catch (Exception ex) {
			}
		}
	}
}