/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

/**
 * Class that generates descrete time marks
 */
public class TimeChannel extends DataChannel {

    private double rate;
    private int size;

    /**
     * Default constructor
     */
    public TimeChannel() {
        this(0.05, 0);
    }

    /**
     * Creates new <code>TimeChannel</code>
     * 
     * @param rate rate in us
     * @param recs number of records in the channel
     */
    public TimeChannel(double rate, int recs) {
        this("Time (ms)", 0);

        this.rate = rate;
        this.size = recs;
    }

    /**
     * Creates new TimeChannel
     * 
     * @param name name of the channel
     * @param recs number of records in the channel
     */
    public TimeChannel(String name, int recs) {
        super(name, 0);
        this.size = recs;
    }

    /**
     * Returns the smallest value in data array
     * 
     * @return lower limit
     */
    public double minLimit() {
        return 0;
    }

    /**
     * Returns the largest value in data array
     * 
     * @return upper limit
     */
    public double maxLimit() {
        return size * rate;
    }

    /**
     * Returns rate of this channel
     * 
     * @return channel rate
     */
    public double getRate() {
        return rate;
    }

    /**
     * Sets rate for this channel
     * 
     * @param rate new channel rate
     */
    public void setRate(double rate) {
        this.rate = rate;
    }

    /**
     * Returns size of the data array
     * 
     * @return size of data array
     */
    public final int size() {
        return size;
    }

    /**
     * Returns value for given index in data array
     * 
     * @param inc index in the array
     * @return data value
     */
    public final double getData(int inc) {
        if (inc > 0) return inc * rate;
        return 0;
    }

    /**
     * Sets value at the given index in data array
     * 
     * @param inc index in the array
     * @param val new data value
     */
    public synchronized final void setData(int inc, double val) {
    }

    /**
     * Grows data array by a specific number of data points
     * 
     * @param inc increment for data array size
     */
    public synchronized void grow(int inc) {
    	size = size + inc;
    }

}