/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.analysis;

import javax.swing.table.DefaultTableModel;

import com.simulogic.dataview.DataDesktop;
import com.simulogic.dataview.DataFrame;
import com.simulogic.dataview.DataGroup;
import com.simulogic.dataview.DataStorage;
import com.simulogic.dataview.DataView;
import com.simulogic.dataview.ResourceManager;

/**
 * Generates dialog box to perform averaging of the number of channels
 */
public class AveragingOptions extends javax.swing.JDialog {

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    private DataDesktop desktop;

    /**
     * Creates new AveragingOptions
     * 
     * @param parent parent frame
     * @param d desktop
     */
    public AveragingOptions(java.awt.Frame parent, DataDesktop d) {
        super(parent, true);
        desktop = d;
        initComponents();
    }

    /**
     * @return the return status of this dialog - one of RET_OK or RET_CANCEL
     */
    public int getReturnStatus() {
        return returnStatus;
    }

    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        viewPanel = new javax.swing.JTabbedPane();
        paramsPanel = new javax.swing.JPanel();
        yPanel = new javax.swing.JPanel();
        ychanLabel = new javax.swing.JLabel();
        ychanTextField = new com.simulogic.utils.IntegerField();
        selectionScrollPane = new javax.swing.JScrollPane();
        selectionTable = new javax.swing.JTable();

        setTitle(ResourceManager.getResource("Channel_Average"));
        addWindowListener(new java.awt.event.WindowAdapter() {

            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog();
            }
        });

        buttonPanel
                .setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        okButton.setText(ResourceManager.getResource("OK"));
        okButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed();
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText(ResourceManager.getResource("Cancel"));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed();
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        paramsPanel.setLayout(new javax.swing.BoxLayout(paramsPanel,
                javax.swing.BoxLayout.Y_AXIS));

        yPanel.setLayout(new java.awt.GridBagLayout());

        yPanel.setBorder(new javax.swing.border.TitledBorder(
                new javax.swing.border.EtchedBorder(), ResourceManager
                        .getResource("Data")));
        ychanLabel.setText(ResourceManager.getResource("Channel_"));
        ychanLabel.setDisplayedMnemonic('c');
        ychanLabel.setLabelFor(ychanTextField);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        yPanel.add(ychanLabel, gridBagConstraints);

        ychanTextField.setText("1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        yPanel.add(ychanTextField, gridBagConstraints);

        paramsPanel.add(yPanel);

        viewPanel
                .addTab(ResourceManager.getResource("Parameters"), paramsPanel);

        selectionScrollPane.setPreferredSize(new java.awt.Dimension(40, 40));
        selectionScrollPane.setViewportView(selectionTable);
        viewPanel.addTab(ResourceManager.getResource("Selection"),
                selectionScrollPane);

        viewPanel.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                if (viewPanel.getSelectedIndex() == 1) {
                    fillSelection();
                }

            }
        });

        getContentPane().add(viewPanel, java.awt.BorderLayout.CENTER);
    }

    private void okButtonActionPerformed() {
        com.simulogic.utils.SwingWorker worker = new com.simulogic.utils.SwingWorker() {
        	DataStorage source;
        	DataGroup result;
        	
            public Object construct() {
                doClose(RET_OK);

                DataFrame ifr = desktop.getActiveFrame();
                if (ifr != null) {
                    DataView view = ifr.getDataContent().getView();
                    source = view.getStorage();

                    try {
                        result = ChannelAvrg.averageTraces(source, ychanTextField
                                .getValue());
                    } catch (Exception e) {
                    }
                }

                return ResourceManager.getResource("Done");
            }

            public void finished() {
            	source.addGroup(result);
                desktop.getTasks().removeTask(
                        ResourceManager.getResource("Channel_Average"));
            }
        };
        desktop.getTasks().addTask(
                ResourceManager.getResource("Channel_Average"));
        worker.start();
    }

    private void fillSelection() {
        DataFrame ifr = desktop.getActiveFrame();
        DataStorage source = ifr.getDataContent().getView().getStorage();

        Object[][] dat = new Object[source.getGroupsSize()][2];

        for (int i = 0; i < source.getGroupsSize(); i++) {
            dat[i][0] = "" + i;
            try {
                if (!source.getChannel(i, ychanTextField.getValue())
                        .getAttribute().isNormal())
                        throw new NullPointerException();
                dat[i][1] = source.getChannel(i, ychanTextField.getValue())
                        .getName();
            } catch (Exception e) {
                dat[i][1] = "--";
            }
        }

        selectionTable.setModel(new DefaultTableModel(dat, new String[] {
                ResourceManager.getResource("Sweep"),
                ResourceManager.getResource("Data")}));
    }

    private void cancelButtonActionPerformed() {
        doClose(RET_CANCEL);
    }

    /** Closes the dialog */
    private void closeDialog() {
        doClose(RET_CANCEL);
    }

    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dispose();
    }

    private javax.swing.JPanel buttonPanel;
    private javax.swing.JPanel yPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JPanel paramsPanel;
    private com.simulogic.utils.IntegerField ychanTextField;
    private javax.swing.JLabel ychanLabel;
    private javax.swing.JTable selectionTable;
    private javax.swing.JScrollPane selectionScrollPane;
    private javax.swing.JTabbedPane viewPanel;

    private int returnStatus = RET_CANCEL;

    /**
     * Overrides setVisible(boolean b) in <code>JDialog</code>
     */
    public void setVisible(boolean b) {
        if (b == true) {
            setResizable(true);
            pack();
            setResizable(false);
            getRootPane().setDefaultButton(okButton);
            setLocationRelativeTo(getOwner());
        }
        super.setVisible(b);
    }

}