/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.legend;

import java.awt.geom.AffineTransform;

import javax.swing.JLabel;
import javax.swing.SwingConstants;

import com.simulogic.dataview.plots.PlotSymbol;

/**
 * Renderer for the plot symbols
 */
public class SymbolBoxRenderer extends JLabel implements
		javax.swing.ListCellRenderer {

	private PlotSymbol symbol = new PlotSymbol(0);

	SymbolBoxRenderer() {
		super();

		this.setHorizontalTextPosition(SwingConstants.TRAILING);
		this.setOpaque(true);

		symbol.setSize(10.0f);
		setPreferredSize(new java.awt.Dimension(30, (int) symbol.getSize() + 4));
	}

	/**
	 * Returns rendering component
	 * 
	 * @param list
	 *            export list
	 * @param value
	 *            current value
	 * @param selection
	 *            true if value is selected
	 * @param row
	 *            current row number
	 * @param hasFocus
	 *            true if value has focus
	 * @return rendering component
	 */
	public java.awt.Component getListCellRendererComponent(
			javax.swing.JList list, Object value, int row, boolean selection,
			boolean hasFocus) {
		if (selection) {
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		} else {
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}

		symbol.setType(((Integer) value).intValue());

		return this;
	}

	/**
	 * Setter for property type.
	 * 
	 * @param type
	 *            New value of property type.
	 */
	public void setType(int type) {
		symbol.setType(type);
	}

	/**
	 * Overrides correspondent JComponent method
	 * 
	 * @param g
	 *            view graphics
	 */
	public void paintComponent(java.awt.Graphics g) {
		super.paintComponent(g);

		double dx = this.getWidth() / 2.0;
		double dy = this.getHeight() / 2.0;

		java.awt.geom.Line2D.Double ld = new java.awt.geom.Line2D.Double();
		java.awt.Graphics2D g2 = (java.awt.Graphics2D) g;
		g2.setStroke(new java.awt.BasicStroke(1.0f));
		g2.setColor(getForeground());

		ld.setLine(0, dy, dx - symbol.getSize(), dy);
		g2.draw(ld);

		try {
			g2.draw(PlotSymbol.symbols[symbol.getType()]
					.createTransformedShape(AffineTransform
							.getTranslateInstance(dx, dy)));
		} catch (Exception e) {
			ld.setLine(dx - symbol.getSize(), dy, dx + symbol.getSize(), dy);
			g2.draw(ld);
		}

		ld.setLine(dx + symbol.getSize(), dy, dx * 2, dy);
		g2.draw(ld);

		g2.dispose();
	}

}