/*
 * Copyright (C) 2000 Sergey Missan. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if any, must
 * include the following acknowledgment: "This product includes software
 * developed by Sergey Missan (http://www.simulogic.com/)." Alternately, this
 * acknowledgment may appear in the software itself, if and wherever such
 * third-party acknowledgments normally appear.
 * 
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SERGEY
 * MISSAN OR simulogic OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NE-
 * GLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.simulogic.utils;

import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

/** Submenu that contains a list of recent file names */
public class RecentFileMenu extends JMenu {
	private Vector names;

	private int maxFiles = 8;

	private Preferences preferences = null;

	private Class actionClass;

	public RecentFileMenu(Class action) {
		preferences = Preferences.userNodeForPackage(action);
		this.actionClass = action;

		init();
	}

	private void init() {
		names = new Vector();
		
		String n = preferences.get(actionClass.getName(), "");
		if ((n != null) && (n.length() > 0)) {
			n = n.substring(1, n.length() - 1);
			String[] list = n.split(",");
			for (int i = 0; i < list.length; i++) {
				names.add(list[i].trim());
			}
		}

		updateMenu();
	}

	private void save() {
		preferences.put(actionClass.getName(), names.toString());
	}

	/**
	 * Adds file name to the menu
	 * 
	 * @param fileName new file name to add
	 */
	public void addFileName(String fileName) {
		names.remove(fileName);
		names.add(0, fileName);
		if (names.size() > maxFiles) {
			names.removeElementAt(maxFiles);
		}
		save();
		updateMenu();
	}

	protected void updateMenu() {
		if ((names == null) || (names.size() == 0)) {
			this.setEnabled(false);
		} else {
			this.setEnabled(true);
		}

		removeAll();
		JMenuItem item;

		for (int i = 0; i < names.size(); i++) {
			try {
				ActionSupport act = (ActionSupport) actionClass.newInstance();
				act.setName((String) names.get(i));
				item = new JMenuItem(act);
				add(item);
			} catch (Exception e) {
			}
		}
	}

}
