<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:cellml="http://www.cellml.org/cellml/1.0#" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.1" xmlns:cmeta="http://www.cellml.org/metadata/1.0#" xmlns:math="http://www.w3.org/1998/Math/MathML" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:fo="http://www.w3.org/1999/XSL/Format">

<!-- Based on Mathml-x stylesheet -->
<!-- CellML support (c) Sergey Missan, 2003 -->

	<xsl:import href="utilities.xsl"/>
	<xsl:import href="linearAlgebra.xsl"/>
	<xsl:import href="statistics.xsl"/>
	<!--Derivatives are taken of functions in differential.xsl.  The math:diff template match is starting point-->
	<xsl:import href="differential.xsl"/>
	<xsl:import href="integral.xsl"/>
	<xsl:import href="complex.xsl"/>

	<xsl:output method="text" encoding="iso-8859-1" indent="yes"/>

	<xsl:strip-space elements="*"/>
	<xsl:param name="packageName"/>
	<xsl:param name="className"/>
	<xsl:param name="baseClassName"/>

	<xsl:template name="description">
	/**
	* <xsl:apply-templates select="/*/rdf:RDF/*[2]/dc:title"/>
	* <xsl:apply-templates select="/*/rdf:RDF/*[2]/cmeta:comment"/>
	*
	* Changes:
	* <xsl:apply-templates select="/*/rdf:RDF/*[1]/cmeta:modification"/>
	*/
	</xsl:template>

	<xsl:template match="cmeta:modification" name="changes">
	* <xsl:value-of select="rdf:value"/>
	* by <xsl:value-of select="cmeta:modifier"/>
	</xsl:template>

	<xsl:template match="cmeta:comment" name="comment">
	* <xsl:value-of select="."/>
	</xsl:template>

	<xsl:template match="dc:title" name="title">
	* <xsl:value-of select="."/>
	</xsl:template>

	<xsl:template match="rdf:RDF" name="rdf" />

	<xsl:template match="cellml:model" name="model">
	// Model: <xsl:value-of select="@name" />

	<xsl:apply-templates/>
	</xsl:template>

	<!--put this function locations in a key lookup table for later access-->
	<xsl:key name="functions" match="math:csymbol" use="translate(concat(., ../*[2]),' &#9;&#10;&#13;','')"/>
	<!--match to the main element-->
	<xsl:template match="math:math" name="math">
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="math:apply" name="apply">
	<xsl:choose>
	    <xsl:when test="./*[position()=1 and name()='sep']">
		<xsl:for-each select="./*[position()&gt;1]">
		<xsl:apply-templates select="."/>;
		</xsl:for-each>
	    </xsl:when>
	    <xsl:when test="position()&lt;3">
		<xsl:apply-templates select="./*[position()=1]"/>
		<xsl:if test="parent::*[name()='math']">;
		</xsl:if>
	    </xsl:when>
	    <xsl:when test="not(../*[position()=1 and local-name()='eq'] and ../*[2]/*[position()=1 and local-name()='ci'])">
		<xsl:apply-templates select="./*[position()=1]"/>
		<xsl:if test="parent::*[name()='math']">;
		</xsl:if>
	    </xsl:when>
	</xsl:choose>
	</xsl:template>

	<xsl:template match="/">
/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * This file was automatically generated from CellML (www.cellml.org) source.
 * This conversion is extremely experimental -- do not attempt to run if you are not a developer!
 *
 */

    package <xsl:value-of select="$packageName"/>;
	<xsl:text>

	import java.io.Serializable;
	import com.javable.cese.templates.*;
	</xsl:text>
	<xsl:call-template name="description"/>
	<xsl:text>
	public class </xsl:text><xsl:value-of select="$className"/><xsl:text> extends </xsl:text><xsl:value-of select="$baseClassName"/><xsl:text> {

	</xsl:text>

	<xsl:call-template name="variables"/>

	<xsl:text>

	/* ** CONSTRUCTOR ** */
	public </xsl:text><xsl:value-of select="$className"/><xsl:text>() {
		super();
		initVariables();
	}

	protected void initVariables() {
	</xsl:text><xsl:call-template name="init-vars"/><xsl:text>
	}

	protected void setComponents() {
	</xsl:text><xsl:call-template name="components"/><xsl:text>
	}

	protected void doSimulate() {
		//compatibility
		V = v;
		dtime = dt;
		time = t;

		setComponents();

		dvdtnew = (V - vnew) / dtime;
		time = time + dtime;
		vnew = V;
		dvdt = dvdtnew;

		// compatibility
		v = V;
		dt = dtime;
		t = time;
	}

	/* Functions that describe the currents begin here */
	</xsl:text>

	<xsl:apply-templates />

    <xsl:text>

    }
</xsl:text>
	</xsl:template>

	<xsl:template match="cellml:component" name="component">
	public void component_<xsl:value-of select="@name"/> () {
		<xsl:apply-templates />
	}
	</xsl:template>

	<xsl:template match="cellml:variable" name="variable">
     	// <xsl:value-of select="@name"/><xsl:if test="not(string-length(@initial_value)=0)">, initial value = <xsl:value-of select="@initial_value"/></xsl:if> (<xsl:value-of select="@units"/>)
     	 <xsl:text>  </xsl:text>
	</xsl:template>

	<xsl:template name="init-vars">
		<xsl:for-each select="node()">
			<xsl:choose>
				<xsl:when test="local-name()='variable'">
				<xsl:if test="not(string-length(@initial_value)=0)">
					<xsl:value-of select="@name"/>=<xsl:value-of select="@initial_value"/>;
				</xsl:if>
				</xsl:when>
				<xsl:otherwise>
				<xsl:call-template name="init-vars" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="components">
		<xsl:for-each select="node()">
			<xsl:choose>
				<xsl:when test="local-name()='component'">
				<xsl:if test="not(string-length(@name)=0)">
					component_<xsl:value-of select="@name"/> ();
				</xsl:if>
				</xsl:when>
				<xsl:otherwise>
				<xsl:call-template name="components" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="capitalize-first-letter">
		<xsl:param name="current-variable" select="string(' ')"/>
		<xsl:copy-of select="translate(substring($current-variable,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
		<xsl:if test="string-length($current-variable)&gt;1">
			<xsl:copy-of select="substring($current-variable,2,string-length($current-variable))"/>
		</xsl:if>
	</xsl:template>

	<xsl:template name="variables">
		<xsl:param name="index" select="1"/>

		<xsl:variable name="variable-list">
			<xsl:for-each select="node()">
				<xsl:choose>
					<xsl:when test="local-name()='ci'">
						<xsl:text> </xsl:text>
						<xsl:choose>
						<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='mn']">
							<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/>
						</xsl:when>
						<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='mi']">
							<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/>
						</xsl:when>
						<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='apply']">
							<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="translate(.,' &#9;&#10;&#13;','')"/>
						</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="get-variables">
							<xsl:with-param name="index" select="$index+1"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</xsl:variable>

		<xsl:variable name="type-list">
			<xsl:for-each select="node()">
				<xsl:choose>
					<xsl:when test="local-name()='ci'">
						<xsl:if test="not(string-length(@type)=0)">
							<xsl:text> </xsl:text>
							<xsl:value-of select="@type"/>
						</xsl:if>
						<xsl:if test="string-length(@type)=0">
							<xsl:text> double</xsl:text>
						</xsl:if>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="get-types">
							<xsl:with-param name="index" select="$index+1"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each><xsl:text> </xsl:text>
		</xsl:variable>

		<xsl:call-template name="define-variables">
			<xsl:with-param name="list" select="substring-after($variable-list, ' ')"/>
			<xsl:with-param name="type-list" select="substring-after($type-list, ' ')"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="get-variables">
		<xsl:param name="index" select="1"/>
		<xsl:for-each select="node()">
			<xsl:choose>
				<xsl:when test="local-name()='ci'">
					<xsl:text> </xsl:text>
					<xsl:value-of select="translate(.,' ','')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="get-variables">
						<xsl:with-param name="index" select="$index+1"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="get-types">
		<xsl:param name="index" select="1"/>
		<xsl:for-each select="node()">
			<xsl:choose>
				<xsl:when test="local-name()='ci'">
					<xsl:if test="not(string-length(@type)=0)">
						<xsl:text> </xsl:text>
						<xsl:value-of select="@type"/>
					</xsl:if>
					<xsl:if test="string-length(@type)=0">
						<xsl:text> double</xsl:text>
					</xsl:if>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="get-types">
						<xsl:with-param name="index" select="$index+1"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="define-variables">
		<xsl:param name="unique-list" select="string(' ')"/>
		<xsl:param name="list" select="string(' ')"/>
		<xsl:param name="type-list" select="string(' ')"/>
		<xsl:param name="index" select="1"/>
		<xsl:param name="current-variable" select="substring-before($list, ' ')"/>
		<xsl:param name="current-type" select="substring-before($type-list, ' ')"/>
		<xsl:variable name="type-Java-type">
			<xsl:choose>
				<xsl:when test="$current-type='matrix'">
					<xsl:text>javax.vecmath.GMatrix</xsl:text>
				</xsl:when>
				<xsl:when test="$current-type='vector'">
					<xsl:text>javax.vecmath.GVector</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>double</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="not(contains($unique-list, $current-variable))">
				<xsl:choose>
					<xsl:when test="$current-type='matrix'">
    protected <xsl:value-of select="$type-Java-type"/>
						<xsl:text> </xsl:text>
						<xsl:value-of select="$current-variable"/>=null;
            </xsl:when>
					<xsl:when test="$current-type='vector'">
    protected <xsl:value-of select="$type-Java-type"/>
						<xsl:text> </xsl:text>
						<xsl:value-of select="$current-variable"/>=null;
            </xsl:when>
					<xsl:otherwise>
    protected <xsl:value-of select="$type-Java-type"/>
						<xsl:text> </xsl:text>
						<xsl:value-of select="$current-variable"/>=0.0;
            </xsl:otherwise>
				</xsl:choose>
    public void set<xsl:call-template name="capitalize-first-letter">
					<xsl:with-param name="current-variable" select="$current-variable"/>
				</xsl:call-template>(<xsl:value-of select="$type-Java-type"/> new<xsl:call-template name="capitalize-first-letter">
					<xsl:with-param name="current-variable" select="$current-variable"/>
				</xsl:call-template>)
    {
        <xsl:value-of select="$current-variable"/>=new<xsl:call-template name="capitalize-first-letter">
					<xsl:with-param name="current-variable" select="$current-variable"/>
				</xsl:call-template>;
    }
    public <xsl:value-of select="$type-Java-type"/> get<xsl:call-template name="capitalize-first-letter">
					<xsl:with-param name="current-variable" select="$current-variable"/>
				</xsl:call-template>()
    {
        return <xsl:value-of select="$current-variable"/>;
    }
			</xsl:when>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="contains(substring-after($list, ' '), ' ') ">
				<xsl:call-template name="define-variables">
					<xsl:with-param name="unique-list" select="concat($unique-list,' ',$current-variable)"/>
					<xsl:with-param name="list" select="substring-after($list, ' ')"/>
					<xsl:with-param name="type-list" select="substring-after($type-list, ' ')"/>
					<xsl:with-param name="index" select="$index+1"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="set-type">
		<xsl:param name="current-type" select="void"/>
		<xsl:choose>
			<xsl:when test="$current-type='matrix'">
				<xsl:text>javax.vecmath.GMatrix</xsl:text>
			</xsl:when>
			<xsl:when test="$current-type='vector'">
				<xsl:text>javax.vecmath.GVector</xsl:text>
			</xsl:when>
			<xsl:when test="$current-type='real[]'">
					<xsl:text>double[]</xsl:text>
				</xsl:when>
			<xsl:when test="$current-type='integer'">
				<xsl:text>int</xsl:text>
			</xsl:when>
			<xsl:when test="$current-type='void'">
				<xsl:text>void</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>double</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="set-argument">
		<xsl:param name="current-variable" select="' '"/>
		<xsl:param name="current-type" select="void"/>
		<xsl:call-template name="set-type">
			<xsl:with-param name="current-type" select="$current-type"/>
		</xsl:call-template><xsl:if test="./*[position()=1 and local-name()='msub']">[]</xsl:if><xsl:text> </xsl:text><xsl:value-of select="$current-variable"/>
	</xsl:template>

	<xsl:template match="math:mrow">
		<xsl:apply-templates select="./*"/>
		<xsl:if test="parent::*[name()='math']">
			<xsl:text>;
    </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="math:semantics">
		<xsl:apply-templates select="./*"/>
		<xsl:if test="parent::*[name()='math']">
			<xsl:text>;
    </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="math:annotation-xml"/>

	<xsl:template match="math:int">
		<xsl:variable name="differential" select="translate(../child::math:bvar, ' ','')"/>
		<xsl:variable name="lower-limit" select="translate(../child::math:lowlimit, ' ','')"/>
		<xsl:variable name="upper-limit" select="translate(../child::math:uplimit, ' ','')"/>
		<!--Example of matching a pattern to replace an integral with the solution(i.e. 1/x from 1.0 to t =&gt; ln(t)-ln(1))
		There are tables of such transforms. Integral and differential; Laplace etc.
		The only question is effiency and making this open source for contributions.-->
		<xsl:if test="../child::math:apply/math:divide/following-sibling::math:cn /following-sibling::math:ci[text()=$differential]">
			<xsl:text>Math.log(</xsl:text>
			<xsl:value-of select="$upper-limit" disable-output-escaping="yes"/>
			<xsl:text>)-Math.log(</xsl:text>
			<xsl:value-of select="$lower-limit" disable-output-escaping="yes"/>
			<xsl:text>);
      </xsl:text>
		</xsl:if>
	</xsl:template>

	<xsl:template match="math:diff">
		<xsl:variable name="differential" select="translate(../child::math:bvar, ' ','')"/>
		// This is an ODE. We should either solve it analytically or plug it into the ODE solver.
		<xsl:apply-templates select="../*[3]"/> = <xsl:apply-templates select="../*[3]"/> + (d<xsl:value-of select="$differential"/> * <xsl:apply-templates select="../../*[3]"/>)
	</xsl:template>

	<xsl:template match="math:sum" name="sum">
		<xsl:variable name="var" select="translate(../child::math:bvar, ' ','')"/>
		<xsl:variable name="lower-limit" select="translate(../child::math:lowlimit, ' ','')"/>
		<xsl:variable name="upper-limit">
		<xsl:apply-templates select="../child::math:uplimit/*"/>
		</xsl:variable>
		<xsl:choose>
		<xsl:when test="../../*[position()=1 and name()='eq']">
		  <xsl:apply-templates select="../../*[2]"/>=0.0;
		  for(double <xsl:value-of select="$var"/>=<xsl:value-of select="$lower-limit"/>;<xsl:value-of select="$var"/>&lt;=<xsl:value-of select="$upper-limit"/>;<xsl:value-of select="$var"/>++)
		  {
		  <!--xsl:for-each select="../child::math:*[5]"-->
		     <xsl:apply-templates select="../../*[2]"/>+=<xsl:apply-templates select="../child::math:*[5]"/>;
		  <!--/xsl:for-each-->
		  }
		</xsl:when>
		<xsl:otherwise>
	  double sum=0.0;
	  for(double <xsl:value-of select="$var"/>=<xsl:value-of select="$lower-limit"/>;<xsl:value-of select="$var"/>&lt;<xsl:value-of select="$upper-limit"/>;<xsl:value-of select="$var"/>++)
	  {
    	 sum+=(<xsl:apply-templates select="../child::math:*[5]"/>);
	  }
	    </xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="math:tendsto" name="tendsto">
            <xsl:variable name="direction" select="@type"/>
            <xsl:choose>
                <xsl:when test="../*[position()=3 and name()='infinity']">
                    <xsl:text>Double.isInfinite(</xsl:text><xsl:apply-templates select="../*[2]"/><xsl:text>)</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="../*[2]"/>
                </xsl:otherwise>
            </xsl:choose>
    </xsl:template>

	<xsl:template match="math:plus" name="plus">
		<xsl:text>(</xsl:text>
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>+</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:quotient" name="quotient">
		<xsl:text>(</xsl:text>
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>)/(</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:minus" name="minus">
		<xsl:text>(</xsl:text>
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>-</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:times" name="times">
		<xsl:text>(</xsl:text>
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>*</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:divide" name="divide">
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>/</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[3]"/>
	</xsl:template>

	<xsl:template match="math:factorial" name="factorial">
		<xsl:text>factorial((int)</xsl:text>
			<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:msup|math:power">
		<xsl:text>Math.pow(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>, </xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[3]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:msubsup[position()=1] /child::math:mo[position()=1 and text()='&#x0222B;']">
		<xsl:variable name="lower-limit" select="../child::math:*[2]"/>
		<xsl:variable name="upper-limit" select="../child::math:*[3]"/>
		<xsl:variable name="differential" select="../following-sibling::math:* //math:*[text()='&#x02146;'] /following-sibling::math:*"/>

    <xsl:value-of select="$upper-limit" disable-output-escaping = "yes"/>
		<xsl:if test="../following-sibling::math:mfrac /child::math:mrow /following-sibling::math:mi[text()=$differential]">
			<xsl:text>Math.log(</xsl:text>
			<xsl:value-of select="$upper-limit" disable-output-escaping="yes"/>
			<xsl:text>)-Math.log(</xsl:text>
			<xsl:value-of select="$lower-limit" disable-output-escaping="yes"/>
			<xsl:text>)
      </xsl:text>
		</xsl:if>
    </xsl:template>

	<xsl:template match="math:mfrac" name="log-integral">
		<xsl:for-each select="./*">
			<xsl:value-of select="name()" disable-output-escaping="yes"/>
			<xsl:text>, </xsl:text>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:msqrt|math:mroot|math:root">
		<xsl:text>Math.sqrt(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:mfrac">
		<xsl:text>(</xsl:text>
		<xsl:for-each select="./child::*">
			<xsl:value-of select="." disable-output-escaping="yes"/>
			<xsl:if test="not(position()>1)">
				<xsl:text>/</xsl:text>
			</xsl:if>
		</xsl:for-each>
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:mo">
		<xsl:choose>
			<xsl:when test="text()='&#x02062;'">
				<xsl:text>*</xsl:text>
			</xsl:when>
			<xsl:when test="text()='+'">
				<xsl:text>+</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="text()" disable-output-escaping="yes"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="math:eq" name="eq">
		<xsl:param name="eqvar" select="'='"/>
		<xsl:choose>
		<xsl:when test="../*[2]/*[position()=1 and local-name()='cn']">
			<xsl:apply-templates select="../*[2]">
			<xsl:with-param name="eqvar" select="' '"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="parent::math:apply/math:apply/*[position()=1 and local-name()='diff']">
			<xsl:apply-templates select="parent::math:apply/*[2]"/>
		</xsl:when>
		<xsl:when test="parent::math:apply/*[position()=3 and local-name()='piecewise']">
			<xsl:apply-templates select="parent::math:apply/*[3]">
			</xsl:apply-templates>
		</xsl:when>
		<xsl:otherwise>
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="position()=2 or (not(position()=1) and not(../../../*[2]/*[position()=1 and local-name()='csymbol']))">
				<xsl:value-of select="$eqvar"/>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
		</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="math:leq" name="leq">
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>&lt;=</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:geq" name="geq">
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>&gt;=</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:lt" name="lt">
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>&lt;</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:gt" name="gt">
		<xsl:for-each select="parent::math:apply/*[position()>1]">
			<xsl:if test="not(position()=1)">
				<xsl:text>&gt;</xsl:text>
			</xsl:if>
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:condition" name="condition">
            <xsl:apply-templates select="./*"/>
    </xsl:template>

	<xsl:template match="math:not" name="not">
            <xsl:text>!</xsl:text><xsl:apply-templates select="../*[2]"/>
    </xsl:template>

	<xsl:template match="math:and" name="and">
                <xsl:text>(</xsl:text>
                <xsl:for-each select="parent::math:apply/*[position()>1]">
                        <xsl:if test="not(position()=1)">
                                <xsl:text> &amp;&amp; </xsl:text>
                        </xsl:if>
                        <xsl:apply-templates select="."/>
                </xsl:for-each>
                <xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template match="math:ci" name="ci">
	<xsl:choose>
	<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='mn']">
		<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/>[(int)<xsl:value-of select="translate(./*[1]/*[2],' ','')"/><xsl:text>]</xsl:text>
	</xsl:when>
	<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='mi']">
		<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/><xsl:value-of select="translate(./*[1]/*[2],' &#9;&#10;&#13;','')"/>
	</xsl:when>
	<xsl:when test="./*[position()=1 and name()='msub'] and ./*[1]/*[position()=1 and name()='mi'] and ./*[1]/*[position()=2 and name()='apply']">
		<xsl:value-of select="translate(./*[1]/*[1],' &#9;&#10;&#13;','')"/>[(int)<xsl:apply-templates select="./*[1]/*[2]"/><xsl:text>]</xsl:text>
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="translate(.,' &#9;&#10;&#13;','')"/>
	</xsl:otherwise>
	</xsl:choose>
	</xsl:template>

	<xsl:template match="math:cn" name="cn">
		<xsl:value-of select="."/>
	</xsl:template>

	<xsl:template match="math:piecewise" name="piecewise">
	<xsl:param name="current-variable" select="../*[2]"/>
		<xsl:for-each select="*">
		<xsl:choose>
		<xsl:when test="local-name()='piece'">
		<xsl:if test="not(position()=1)"><xsl:text>else </xsl:text></xsl:if>if(<xsl:apply-templates select="./*[2]"/>)
	{
		<xsl:value-of select="$current-variable"/>=<xsl:apply-templates select="./math:*[1]"/>;
	}
		</xsl:when>
		<xsl:when test="local-name()='otherwise'">
	else
	{
		<xsl:value-of select="$current-variable"/>=<xsl:apply-templates select="./math:*[1]"/>;
	}
		</xsl:when>
		</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="math:pi">
		<xsl:text>Math.PI</xsl:text>
	</xsl:template>
	<xsl:template match="math:exp">
		<xsl:variable name="variable" select="parent::math:apply/*[2]"/>
		<xsl:text>Math.exp(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:ln">
		<xsl:variable name="variable" select="parent::math:apply/*[2]"/>
		<xsl:text>Math.log(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:log">
	    <xsl:variable name="variable" select="parent::math:apply/*[3]"/>
		<xsl:variable name="base" select="parent::apply/*[2]"/>
		<xsl:text>Math.log(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[3]"/>
		<xsl:text>)/Math.log(</xsl:text>
		<xsl:value-of select="$base"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:sin">
		<xsl:variable name="variable" select="parent::math:apply/*[2]"/>
		<xsl:text>Math.sin(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:cos">
		<xsl:variable name="variable" select="parent::math:apply/*[2]"/>
		<xsl:text>Math.cos(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:tan">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.tan(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:sec">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.sec(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:csc">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.csc(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:cot">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.cot(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:sinh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.sinh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:cosh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.cosh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:tanh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.tanh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:sech">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.sech(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:csch">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.csch(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:coth">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.coth(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arcsin">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.asin(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccos">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.acos(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:atan">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.atan(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccosh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arccosh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccot">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arccot(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccoth">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arccoth(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccsc">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arccsc(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arccsch">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arccsch(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arcsec">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arcsec(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arcsech">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arcsech(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arcsinh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arcsinh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
	<xsl:template match="math:arctanh">
		<xsl:variable name="variable" select="parent::apply/*[2]"/>
		<xsl:text>Math.arctanh(</xsl:text>
		<xsl:apply-templates select="parent::math:apply/*[2]"/>
		<xsl:text>)</xsl:text>
	</xsl:template>
</xsl:stylesheet>
